;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Loading this file installs CG/JS.
;; Please see readme.txt for more information
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(in-package :user)

(eval-when (compile eval load)
  (require :shell)
  (use-package :excl.shell))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; validate files

(let ((platform-file
       (directory
	(merge-pathnames
	 "PLATFORM-*"
	 (path-pathname *load-pathname*)))))
  (or platform-file
      (error "Could not find the PLATFORM file."))
  
  (when (> (length platform-file) 1)
    (error "There is more than one PLATFORM file: ~a." platform-file))
  (setq platform-file (car platform-file))
  
  (or (=~ "^PLATFORM-(.*)" (file-namestring platform-file))
      (error "Cannot parse the PLATFORM file."))
  
  (let ((expected-platform $1)
	(current-platform
	 (format nil "~a~a" (symbol-name sys::.os.) #+smp "smp" #-smp "")))
    (or (string-equal expected-platform current-platform)
	(error
	 "The current platform (~a) does not match the downloaded version (~a)."
	 current-platform expected-platform)))
  ;; We are on the correct platform, fall through...
  )

;; HTML files needed for the operation of CG/JS
;; (this variable's value is generated by build_cgjs.sh)
(defvar *html-files* '("cgjs.html" "ssl-not-found.html" "browser-not-supported.html" ))

(defvar *extra-files*)
(setq *extra-files* nil)

(dolist (file *html-files*)
  (setq file (merge-pathnames file *load-pathname*))
  (or (probe-file file)
      (error "~a does not exist." file))
  (push file *extra-files*))
;; All *html-files* exist...

;; These two are directories but the pathname is specified as
;; a file, so merging works below.
(defvar *cg2js-directory* (merge-pathnames "code.cg2js" *load-pathname*))
(defvar *idejs-directory* (merge-pathnames "code.idejs" *load-pathname*))

(dolist (d (list *cg2js-directory* *idejs-directory*))
  (or (probe-file d)
      (error "~a does not exist." d)))

;; MD5 checksums of the incoming files
;; (this variable's value is generated by build_cgjs.sh)
(defvar *checksums* 
'(
(#x94fc7a8dd9bdd5aa2a83bc06a8cde00a . "./code.cg2js/cg-acache.fasl")
(#x8569868b9cd640be6780b8c84062d5f9 . "./code.cg2js/cg-base.fasl")
(#x395253c77b458fb2c8f3587548fd932e . "./code.cg2js/cg-bitmap-pane-clipboard.fasl")
(#xea45858fcf71616ec9b9158deb9e77b6 . "./code.cg2js/cg-bitmap-pane.fasl")
(#x9030e1e0111a552b741980cab0e58d16 . "./code.cg2js/cg-bitmap-stream.fasl")
(#x9cd1f42cf57a04032fa7632ad76b1d8f . "./code.cg2js/cg-button.fasl")
(#x0d3081c8645e8c84014a742263f3f2a6 . "./code.cg2js/cg-calendar.fasl")
(#xffcbb46b3a4fc2b027cf201b6187d2a1 . "./code.cg2js/cg-caret.fasl")
(#x6bac951424ecffb9b5d57f29940848c1 . "./code.cg2js/cg-chart-or-plot.fasl")
(#xa5f7ba19d063dd4468b77ad9d1de2c2b . "./code.cg2js/cg-chart-widget.fasl")
(#xa77ee110555c6cfc9eaf934fab895569 . "./code.cg2js/cg-check-box.fasl")
(#x7bc2d8d7a6e1f96ec49c3c7dd279eefc . "./code.cg2js/cg-choice-list.fasl")
(#x139a48d9083fe953c983d46176eb800a . "./code.cg2js/cg-choose-printer.fasl")
(#xf109e1ee4f9ab5f82af753bffa6e10a2 . "./code.cg2js/cg-class-grid.fasl")
(#xda53e35be4182de55cddbfda76b31327 . "./code.cg2js/cg-class-slot-grid.fasl")
(#xc666e2e8fa21eafa55d8c4be9a2c158d . "./code.cg2js/cg-class-support.fasl")
(#x5ee7c740f1dc4f09c5ca6353debbe727 . "./code.cg2js/cg-clipboard-pixmap.fasl")
(#x8c90caf91e69e0e065f4bdf2ecdd7df7 . "./code.cg2js/cg-clipboard-stack.fasl")
(#x90108c245c9f0986e58eaa25e91bedbc . "./code.cg2js/cg-clipboard.fasl")
(#x2f1907d1d5b694b23794e1e603929fff . "./code.cg2js/cg-color-dialog.fasl")
(#x2763f77d2e12e949895d2c1339abfb58 . "./code.cg2js/cg-combo-box.fasl")
(#xf2c7450538bd76ebf1e43b2bbf1dd542 . "./code.cg2js/cg-common-control.fasl")
(#xf15c33de15b7cc36ee18c62ca7338911 . "./code.cg2js/cg-comtab.fasl")
(#x1d670d8357a6fb372f5d0c9574e569c6 . "./code.cg2js/cg-cursor-pixmap.fasl")
(#x255f4337ed3055bb4be1307aaf118a52 . "./code.cg2js/cg-curve.fasl")
(#x2ebf68922c8d246f3a704d6ec8f7d710 . "./code.cg2js/cg-dde-utils.fasl")
(#x3b508ed0a87f9b45b601d1ab4b25bc94 . "./code.cg2js/cg-dev.fasl")
(#x22d4aa066bd4c8b20c170ee96bdd84f1 . "./code.cg2js/cg-dialog-item.fasl")
(#x27a684766d664b82dfc7d3131fbc05bd . "./code.cg2js/cg-directory-dialog-os.fasl")
(#x6cdc3640ea7563fdee5cd3e882071173 . "./code.cg2js/cg-directory-dialog.fasl")
(#x6bae996f626cd676de72e69ca7f5f176 . "./code.cg2js/cg-drag-and-drop-image.fasl")
(#xb27ba2595aae184a09248e2ac0c75b31 . "./code.cg2js/cg-drag-and-drop.fasl")
(#x90a3e72f91ea6a4b2a3908d6faebb80e . "./code.cg2js/cg-drawable-clipboard.fasl")
(#x53b22f005d7e6bdedc3bcd65852f0fa0 . "./code.cg2js/cg-drawable.fasl")
(#xa01a8a2426b2669c21a2c9759cd0c5ee . "./code.cg2js/cg-dropping-outline.fasl")
(#xcd7a5dc4f89c336bce8307aa6c83febe . "./code.cg2js/cg-edit-in-place.fasl")
(#x89690eecb736ddbe61b1d19f3d687e64 . "./code.cg2js/cg-editable-text.fasl")
(#x0d928a675995c397778da82efe2e7cb2 . "./code.cg2js/cg-file-dialog.fasl")
(#xae548830f5d26abae83e96d59b0e6244 . "./code.cg2js/cg-fill-texture.fasl")
(#x91dcb30debbbb1ea75cc4ef42958f7e0 . "./code.cg2js/cg-find-string-dialog.fasl")
(#xe448d989bd08fd9391205827bf1017bd . "./code.cg2js/cg-font-dialog.fasl")
(#x3c6544605432c6e412eb263813b7bcd6 . "./code.cg2js/cg-gesture-emulation.fasl")
(#x6d8a22eb8f00fce5c77f7b2d1c5d5ef3 . "./code.cg2js/cg-get-pixmap.fasl")
(#xbfa644c49e34322706cae529d61b116a . "./code.cg2js/cg-get-position.fasl")
(#x371dc047c9e1c1048dcb0527e0fbdf68 . "./code.cg2js/cg-graphics-context.fasl")
(#xa8ac140dd8c37a01bb131e5bfa6fb91e . "./code.cg2js/cg-grid-widget-drag-and-drop.fasl")
(#xf576a890b49a84a4aafa70bbdf4a35ad . "./code.cg2js/cg-grid-widget.fasl")
(#x14620e196d3b7fd1ff7e9ae1cde1f568 . "./code.cg2js/cg-group-box.fasl")
(#x74aeb2c04f872ac5527aeefc4da905ec . "./code.cg2js/cg-header-control.fasl")
(#x48de898700a3e574fc23809c01a03e59 . "./code.cg2js/cg-hotspot.fasl")
(#x369b21e0c175953a4cc7fa023c49bc3c . "./code.cg2js/cg-html-dialog.fasl")
(#x0e169e8563274193cac3f382a1a15734 . "./code.cg2js/cg-html-widget.fasl")
(#x7b72be36969d56e0a31166609a0f049a . "./code.cg2js/cg-icon-pixmap.fasl")
(#x3f2a1f0cd5699e30ee636deb6c30c134 . "./code.cg2js/cg-icon.fasl")
(#x416761ffa2febfb6c8b12f24f0c433d0 . "./code.cg2js/cg-ie.fasl")
(#x19b4c50aee59902d083be925f50716f2 . "./code.cg2js/cg-intersect-posbox.fasl")
(#x8f5dfd2be84d49650add513179748d36 . "./code.cg2js/cg-intersect.fasl")
(#xdc2370ce53e4a1d77c6c3e073baea535 . "./code.cg2js/cg-item-list.fasl")
(#xdbb48414b5658e19fedc2a82e434343c . "./code.cg2js/cg-keyboard-shortcuts.fasl")
(#x4f5def0786623757d92f3b55d9f9c2bd . "./code.cg2js/cg-lamp.fasl")
(#xeba63c0fac77f6634da1afd3333461d6 . "./code.cg2js/cg-layout.fasl")
(#x81505b5be7257f1a0061b2a435dbf10b . "./code.cg2js/cg-lettered-menu.fasl")
(#x76b78005049eeb1826aff41461bb74e9 . "./code.cg2js/cg-lisp-edit-pane.fasl")
(#x81eb154573e3fa39c04e564cc544d529 . "./code.cg2js/cg-lisp-text.fasl")
(#xadd94c4e2358771e8e4887f56eb2eba9 . "./code.cg2js/cg-lisp-widget.fasl")
(#x04a488c3df6a2c890ed18a82dbf40c01 . "./code.cg2js/cg-list-view.fasl")
(#x1610ae8ffcd114673311c352c2ae6733 . "./code.cg2js/cg-mci.fasl")
(#x81dd6fc032b63f7995bc6008923f3fe2 . "./code.cg2js/cg-menu-tooltip.fasl")
(#xd000cf27f7bbf9d27785e608660a0bd2 . "./code.cg2js/cg-menu.fasl")
(#xfdfc866821d02514413ba9ac1d932cff . "./code.cg2js/cg-message-dialog.fasl")
(#xeb903c0e20d4a65ced0584f00d84b593 . "./code.cg2js/cg-multi-line-editable-text.fasl")
(#xb777ff3aa8cd30b3547338da9a775967 . "./code.cg2js/cg-multi-line-lisp-text.fasl")
(#x7a1de601a6b82a137624b690ba98252b . "./code.cg2js/cg-multi-picture-button-drag-and-drop.fasl")
(#xc9f09068674a5258a3d3ada884d1fc94 . "./code.cg2js/cg-multi-picture-button-tooltip.fasl")
(#x6fba568c539150b3f2c06d9f88534540 . "./code.cg2js/cg-multi-picture-button.fasl")
(#x40764d72aa25162dd061a9f56cf6f43a . "./code.cg2js/cg-nodes.fasl")
(#x818e4d367e0a8405f08802476be7e207 . "./code.cg2js/cg-object-editor-layout.fasl")
(#x9f2e18307e1ffb4da6726c09a0dc0ac7 . "./code.cg2js/cg-object-editor.fasl")
(#xbdc93fb7905cfe7d6a84b14a0562ce50 . "./code.cg2js/cg-ocx.fasl")
(#x7c475102953edaa65b4e43e505270dd3 . "./code.cg2js/cg-os-widget.fasl")
(#x675088404ec7d6ca0b12711c6f3c383a . "./code.cg2js/cg-os-window.fasl")
(#x71c361101f54912ca8f012b953289812 . "./code.cg2js/cg-outline-drag-and-drop.fasl")
(#x4a6a253e3d55af5c1f3de6e7884667ec . "./code.cg2js/cg-outline-edit-in-place.fasl")
(#xa54c1421a2f93e16a0dfc1bd376edb01 . "./code.cg2js/cg-outline.fasl")
(#xf6a07bcf1f508148f66d09ee62855e9c . "./code.cg2js/cg-palette.fasl")
(#xbe0e874a044bd56df0ee93bae7939600 . "./code.cg2js/cg-paren-matching.fasl")
(#x8f3007cdb67381aabfa0d543fd2acc00 . "./code.cg2js/cg-picture-widget-palette.fasl")
(#xd316c6f2e770d79b2e6a828b33d4905a . "./code.cg2js/cg-picture-widget.fasl")
(#xe807a047b5ed90c2e9f937ebe40e8dcd . "./code.cg2js/cg-pixmap-file-io.fasl")
(#xedd5b7163898e923c704e38864e6a88c . "./code.cg2js/cg-pixmap-printing.fasl")
(#xe2a6f052772927f5ef3ec49c3b5cd645 . "./code.cg2js/cg-pixmap-rotate.fasl")
(#x69edd7d3c0ae5d26ec4938e51c8c924b . "./code.cg2js/cg-pixmap-widget.fasl")
(#x0f65f68bd8016341689ef7c94c8e7131 . "./code.cg2js/cg-pixmap.fasl")
(#x2efed9537c7da18428022503af3a38ef . "./code.cg2js/cg-plot-widget.fasl")
(#x660dbf20d7b1fe41ac58cf66216baf33 . "./code.cg2js/cg-printing.fasl")
(#x69df24852471f27d3f99729ff7e89eda . "./code.cg2js/cg-progress-indicator.fasl")
(#x3aea1b0cbd8ad9358761137ed38f8ab8 . "./code.cg2js/cg-project-window.fasl")
(#x7e54e292651ce4d42295eafc352a058d . "./code.cg2js/cg-property.fasl")
(#xbb489535f32e1c60f6e22c1aa5038502 . "./code.cg2js/cg-radio-button.fasl")
(#xb8ce5aadbb4aaf71e08f21ed50095558 . "./code.cg2js/cg-rich-edit-pane-clipboard.fasl")
(#xa9e797aa7b3ad27b37c6521bb9608017 . "./code.cg2js/cg-rich-edit-pane-printing.fasl")
(#x5c6871352614a54ad3b8e10c9c6e7eb5 . "./code.cg2js/cg-rich-edit-pane.fasl")
(#xf7eac60ad138b439848db91ddbb6a502 . "./code.cg2js/cg-rich-edit.fasl")
(#xa0f5db8d4f137f6f034d4a702f64ec48 . "./code.cg2js/cg-sample-file-menu.fasl")
(#x62880e61555a6daf45e9e61f39e5f35e . "./code.cg2js/cg-scaling-stream.fasl")
(#xe468fd20c9674f1d05b6fc04f4759764 . "./code.cg2js/cg-scroll-bar-mixin.fasl")
(#x9330b574352d9a6f76869d9aac495e86 . "./code.cg2js/cg-scroll-bar.fasl")
(#xc85af458f7b4fdddd32738d5529731b8 . "./code.cg2js/cg-scrolling-static-text.fasl")
(#x542530b4f29c22691a772290a900a786 . "./code.cg2js/cg-selected-object.fasl")
(#x880d79704eceb18e05fe36da83a18d9d . "./code.cg2js/cg-shortcut-menu.fasl")
(#xfe6309fae7c8e948b2cb677c29f2c6dd . "./code.cg2js/cg-split-bar.fasl")
(#x204bdf1e157a8a34c2566d0cd41273cb . "./code.cg2js/cg-static-text.fasl")
(#x53cffc8589e34c707651c97ea13d9697 . "./code.cg2js/cg-status-bar.fasl")
(#x84f5a3ef7705eaa782db68e045f1bf51 . "./code.cg2js/cg-string-dialog.fasl")
(#x407814f1a4d415db923c5c1d1913136f . "./code.cg2js/cg-tab-control.fasl")
(#x7776f78d4b1d11e380c5167b3bfb03b9 . "./code.cg2js/cg-template-string.fasl")
(#x9eb383be1cf401aeffb97807eb56012a . "./code.cg2js/cg-text-edit-pane-file-io.fasl")
(#xe9238547ca61eb71ff61f5b5c4bd4d9d . "./code.cg2js/cg-text-edit-pane-mark.fasl")
(#x01a653a0319b718d832f5aeda36c118d . "./code.cg2js/cg-text-edit-pane.fasl")
(#x7dc6dc73313c1bfaaa038ff3da72eeaf . "./code.cg2js/cg-text-or-combo.fasl")
(#x28cfa96dc62819a4e517530757efb388 . "./code.cg2js/cg-text-widget.fasl")
(#x9b44d6ebfd4c8e7e2a5edf4abf29790e . "./code.cg2js/cg-timer.fasl")
(#xa3e3c34c09dd2ea9d0d2fcd07edee426 . "./code.cg2js/cg-toggling-widget.fasl")
(#x86fcf3dcf1c80c8d65a223f8896c9451 . "./code.cg2js/cg-toolbar.fasl")
(#x24da2d4b0475c3f8854d990931c9480d . "./code.cg2js/cg-tooltip.fasl")
(#xb40f55e741d535697923a3e7e2866c75 . "./code.cg2js/cg-touch.fasl")
(#x6d26b361abb19a05f998b97b81350094 . "./code.cg2js/cg-trackbar.fasl")
(#xc9733419dfbae6ccd44165d680e21f0f . "./code.cg2js/cg-tray.fasl")
(#x32bf99549d3b3eb8501d24c4ab55eb6e . "./code.cg2js/cg-up-down-control.fasl")
(#x9c7fb7d2ac00e31c909c0d980e46ba82 . "./code.cg2js/cg-utility-dialog.fasl")
(#x637a3a5560822d10e2c2876a457fd912 . "./code.cg2js/cg-web-browser-dde.fasl")
(#x4606e8c2896237de23d3c0a1b768f379 . "./code.cg2js/cg-web-browser.fasl")
(#x49d3f4d6efc060a97af52900fad02720 . "./code.cg2js/cg-wrap-string.fasl")
(#x85a9b4e619979f77f82169a4feca0544 . "./code.cg2js/cg-yes-no-list.fasl")
(#xc3f9282655bfb08fddb7e2c5190155e1 . "./code.cg2js/cg-yes-no-string.fasl")
(#x6bf6c039ae24b9fb41e3cc4f3bf3169d . "./code.cg2js/cg.fasl")
(#x3af9ee89d603a8f3cd57db758d35f4d7 . "./code.idejs/ide-acache.fasl")
(#x56bdaaeb3ac25c5a3433238dcaf7d922 . "./code.idejs/ide-apropos.fasl")
(#x20d0eac2f5915175aebbf8839fcdfed1 . "./code.idejs/ide-base.fasl")
(#xfb0123830e84f0797ccba2a1ae26c6c6 . "./code.idejs/ide-class-browser.fasl")
(#xbc55fc524d1b0ed41a96e63a1a17f22d . "./code.idejs/ide-class-grapher.fasl")
(#x8da2532bc8328686601771c034d3edb8 . "./code.idejs/ide-clipboard.fasl")
(#xdb1beb4bec0a4cb2c416ab92b0d3d925 . "./code.idejs/ide-dev.fasl")
(#x405ddd0350adb29fb95114990b80a1a2 . "./code.idejs/ide-editor.fasl")
(#x54ef79f14e936c4b3454c8e47ab490ea . "./code.idejs/ide-find-definitions.fasl")
(#xd951aef79924c94681dd1fccf505dc11 . "./code.idejs/ide-find-in-files.fasl")
(#xf26ce514c50c95d844b7ddb52ed45d0c . "./code.idejs/ide-form.fasl")
(#xa14065e5f56e5394ac91a25737199826 . "./code.idejs/ide-grapher.fasl")
(#xad0f30291065d4f7f42ea51e9677d0d1 . "./code.idejs/ide-inspector.fasl")
(#x96c47027971338cb37f1760caa6b0ab6 . "./code.idejs/ide-intro.fasl")
(#x2942062f29fad7ea21630500a556e2b4 . "./code.idejs/ide-keyshort.fasl")
(#xcc9d6351f0763d0cb04a2af768739e4d . "./code.idejs/ide-navigator.fasl")
(#x9f84a5fa538a70de8c49a246a9d50fbb . "./code.idejs/ide-package-list.fasl")
(#xdd6fc45ba74a033bcad4753e763dd386 . "./code.idejs/ide-patch-dialog.fasl")
(#xc35d6e99099304504637256fe2d41b56 . "./code.idejs/ide-process-browser.fasl")
(#x65193f70b99f96d5e6ba649946541270 . "./code.idejs/ide-project.fasl")
(#x9d2e3e0555e7980ebcf8d5b4f1fdde44 . "./code.idejs/ide-trace.fasl")
(#xadf03209139a4668af0015daedaf3a5c . "./code.idejs/ide-window-list.fasl")
(#x9e563d6de41c9babed3f255503ec4bbf . "./code.idejs/ide.fasl")
(#xdf73f3983b983ba8030e7ee4702b498a . "./cgjs.html")
(#xa0fd27890288fb85b9ee621ed47b2251 . "./ssl-not-found.html")
(#x4e1b21622d7d9cc4e6fc09c7875f8430 . "./browser-not-supported.html"))
 )

(let ((files-ok
       (dolist (checksum *checksums* t)
	 (when (/= (car checksum)
		   (md5-file
		    (merge-pathnames (cdr checksum) *load-pathname*)))
	   (return nil)))))
  (cond
   ((not files-ok)
    (error "The extracted files failed the checksum test.  ~
Please download and/or extract again."))
   (t
    (format t "Checksums on files are OK...~%"))))

;; We have validated all the files are there...

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; check for needed patches

(eval-when (compile eval load) (require :build))

(defvar *required-patches* '(#+macosx ("build" . 3)))

(dolist (required-patch *required-patches*)
  (let ((existing-patch (assoc (car required-patch)
			    (cdr (assoc :system sys::*patches*))
			    :test #'string=)))
    (when (null existing-patch)
      (error "Could not find patch info for the ~s module."
	     (car required-patch)))
    (when (not (>= (second existing-patch) (cdr required-patch)))
      ;; The build patch number is NOT >= to the one we require
      (error "~
Expected version ~d of ~s but have version ~d.  Please update ~
your lisp with update.~a and then do the CG/JS installation again."
	     (cdr required-patch) (car required-patch)
	     (second existing-patch)
	     #+mswindows "exe"
	     #-mswindows "sh"))))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; copy files

(defun copy (source dest)
  (cond
   ((file-directory-p source)
    (format t "Copy directory ~a~%" (enough-namestring source))
    (copy-directory source dest :quiet t))
   (t
    (format t "Copy file ~a~%" (enough-namestring source))
    (sys:copy-file source dest :verbose nil))))

(defvar *sys* (translate-logical-pathname (pathname "sys:")))

(cond
 ((excl::pathname-equalp (truename *sys*)
			 (path-pathname *load-truename*))
  (format t "Files are already in place, no copying requiring.~%"))
 (t ;; copy files, replacing them with new ones if they already exist
  (let (dest)
    (dolist (from *extra-files*)
      (setq dest (merge-pathnames (file-namestring from) *sys*))
      (when (probe-file dest) (delete-file dest))
      (copy from dest))
    
    (dolist (from (list *cg2js-directory* *idejs-directory*))
      (setq dest (merge-pathnames (file-namestring from) *sys*))
      ;; now make it a directory pathname
      (setq dest (pathname-as-directory dest))

      (when (probe-file dest) (delete-directory-and-files dest))
      (copy (pathname-as-directory from) dest)))))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; update develenv.cl to load CG/JS before the original version of CG

(defvar *setup-cgjs-loading* "sys:setup-cgjs-loading.cl")

;; Because users might use an ANSI lisp to install, have to make sure
;; the resulting file can be loaded into an ANSI and Modern lisp.
(with-open-file (s *setup-cgjs-loading* :direction :output
		 :if-exists :supersede)
  (format s "~
;; This file was created by install-cgjs.cl.
;; Please do not edit it.

")
  (dolist (dir (list "code.cg2js/" "code.idejs/"))
    (format s "
(let ((path (make-pathname
	     :type \"fasl\"
	     :directory (pathname-directory
			 (merge-pathnames
			  ~s
			  (translate-logical-pathname \"sys:\"))))))
  (format t \";; *require-search-list*: adding: ~~s~%\" path)
  (push path sys:*require-search-list*))

"
	    dir)))

(defvar *develenv.cl* (pathname "sys:develenv.cl"))

(when (not (probe-file *develenv.cl*))
  (setf (file-contents *develenv.cl*)
    ";; This file is loaded before images are rebuilt with update.sh/update.exe

"))

(let ((contents (file-contents *develenv.cl*)))
  (cond
   ((match-re
     (format nil "^#\\|\\|# \\(load \"sys:~a"
	     (file-namestring *setup-cgjs-loading*))
     contents
     :multiple-lines t)
    (format t "develenv.cl already correctly loads CG/JS.~%"))
   ((match-re
     (format nil "^#\\+ignore \\(load \"sys:~a"
	     (file-namestring *setup-cgjs-loading*))
     contents
     :multiple-lines t)
    (format t "develenv.cl: CG/JS disabled; will enable.~%")
    (setf (file-contents *develenv.cl*)
      (replace-re  contents
		  (format nil "^#\\+ignore *\\(load \"sys:~a"
			  (file-namestring *setup-cgjs-loading*))
		  (format nil "#||# (load \"sys:~a"
			  (file-namestring *setup-cgjs-loading*))
		  :multiple-lines t
		  :count 1)))
   (t
    (format t "develenv.cl: adding CG/JS initialization.~%")
    (setf (file-contents *develenv.cl*)
      (concatenate 'simple-string
	contents
	(format nil "~

;; To disable the use of CG/JS and go back to the originally released
;; (and possibly patched) version of CG/IDE, change the #||# to
;; #+ignore on the following line and the rebuild images by running
;; update.sh or update.exe, depending on your platform:
#||# (load \"sys:~a\")~%~%"
		(file-namestring *setup-cgjs-loading*)))))))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; success!

(let ((msg (format nil "~
CG/JS installation successful!

NOTE: to use CG/JS you must rebuild images by running 'update.~a'
      in the Allegro CL directory (~a)."
		   #+mswindows "exe"
		   #-mswindows "sh"
		   *sys*)))
  #-mswindows (format t "~&~%~a~%" msg)
  #+mswindows (excl::internal-message-box msg "CG/JS Installation" :type :ok))
